<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class FtpProcess
 * @package FiloBlu\MappConnector\Console\Command
 */
class FtpProcess extends Command
{

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \FiloBlu\MappConnector\Model\MappCSV
     */
    protected $_mappCSV;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_loggerHelper;

    /**
     * @var array
     */
    protected $_entities = ['customer','subscriber','order','product','quote'];

    /**
     * FtpProcess constructor.
     * @param \FiloBlu\MappConnector\Model\Config $config
     * @param \FiloBlu\MappConnector\Model\MappCSV $mappCSV
     * @param \Magento\Framework\App\State $state
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Config $config,
        \FiloBlu\MappConnector\Model\MappCSV $mappCSV,
        \Magento\Framework\App\State $state,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
    )
    {
        $this->_config = $config;
        $this->_mappCSV = $mappCSV;
        $this->_state = $state;
        $this->_loggerHelper = $loggerHelper;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    )
    {
        //Area Code Fix
        try {
            $this->_state->setAreaCode(\Magento\Framework\App\Area::AREA_GLOBAL);
        }
        catch (\Exception $e) {
            //Area code is already set
        }

        $entity = strtolower($input->getArgument('entity'));

        //If entity is specified and exist process entity
        if(!empty($entity)) {
            if(in_array($entity, $this->_entities)) {
                $this->_mappCSV->setJobType($entity);
                $this->_mappCSV->makeCSVAndSend();
            }
            else {
                $output->writeln("WARNING: The entity specified ".$entity." doesn't exists");
            }
        }
        // Else process all entities
        else {
            foreach ($this->_entities as $entity_item) {
                $this->_mappCSV->setJobType($entity_item);
                $this->_mappCSV->makeCSVAndSend();
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("mapp:ftp:process");
        $this->setDescription("Process waiting Mapp FTP jobs");
        // Next line will add a new required parameter to our script
        $this->addArgument('entity', InputArgument::OPTIONAL, __('Please specify an Entity'));
        parent::configure();
    }
}
