<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

/**
 * Class OrderItem
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class OrderItem extends \FiloBlu\MappConnector\Model\From\Entity\AbstractEntity implements \FiloBlu\MappConnector\Model\From\Entity\EntityInterface  {

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'order_item';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Sales\Model\Order\Item';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'order_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'order_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = '\FiloBlu\MappConnector\Model\To\Mapp\FTP';

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'order_id', 'primaryMapp' => 'email'];

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/order_mapping/order_item';

    /**
     * @var
     */
    protected $_order;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $_orderInterface;

    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $_product;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Eav\Api\AttributeSetRepositoryInterface
     */
    protected $_attributeSet;

    /**
     * OrderItem constructor.
     * @param Order $order
     * @param \FiloBlu\MappConnector\Model\Config $config
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\From\Entity\Order $order,
        \FiloBlu\MappConnector\Model\Config $config,
        \Magento\Sales\Api\Data\OrderInterface $orderInterface,
        \Magento\Catalog\Model\Product $product,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLoggerHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet
    )
    {
        $this->_config = $config;
        $this->_order = $order;
        $this->_orderInterface = $orderInterface;
        $this->_product = $product;
        $this->_fileLogger = $fileLoggerHelper;
        $this->_storeManager = $storeManager;
        $this->_attributeSet = $attributeSet;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getDefaultData($record){
        $result = [];

        //Get Store
        $store = $this->_storeManager->getStore($record['store_id']);

        //Get Order
        $order = $this->_orderInterface->load($record['order_id']);

        //Get Product
        $product = $this->_product->load($record['product_id']);

        //Get Product Thumbnail
        $thumbnail = ($product->getThumbnail() === 'no_selection') ? '' : $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getThumbnail();

        //Get Attribute Set Name
        $attributeSetId = $product->getAttributeSetId();
        $attributeSetName = '';
        if(!empty($attributeSetId)) {
            $attributeSet = $this->_attributeSet->get($attributeSetId);
            $attributeSetName = $attributeSet->getAttributeSetName();
        }

        // Order link
        $orderLink = $store->getUrl('sales/order/view', array('order_id' => $record['order_id']));

        // Invoice download link
        $invoiceDlLink = $store->getUrl('sales/order/printInvoice', array('order_id' => $record['order_id']));

        // Populate $result with mandatory data
        $result['Key'] = $order->getCustomerEmail();
        $result['OrderId'] = $record['order_id'];
        $result['StoreId'] = $record['store_id'];
        $result['UniqueOrderItemID'] = $record['item_id'];
        $result['SKU'] = $record['sku'];
        $result['ProductURI'] = $product->getProductUrl();
        //TODO: Get Category of product
        $result['ProductCategory'] = $product->getCategory();
        $result['ProductOptions'] = $this->getFormattedOptions($this->_config->unserializeConfigData($record['product_options']));
        $result['ProductMainThumbnail'] = $thumbnail;
        $result['Description'] = $product->getShortDescription();
        $result['AttributeSet'] = $attributeSetName;
        $result['OrderLink'] = $orderLink;
        $result['InvoiceDownloadLink'] = $invoiceDlLink;
        $result['ShippingAddress'] = $this->getFormattedAddress($order->getShippingAddress());
        $result['InvoicingAddress'] = $this->getFormattedAddress($order->getBillingAddress());

        return $result;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary,$id)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('sales_order_item'); //gives table name with prefix

        //Select Data from table
        $sql = "Select * FROM " . $tableName.' WHERE '.$primary.' = '.$id ;
        $result = $connection->fetchAll($sql);

        return $result;
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record){
        $result = [];

        //Prendo le informazioni dai moduli correlati
        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_order,$record['order_id'])
        );

        return $result;
    }

    /**
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        if($type == 'full') {
            //Full
            return $this->_order->getModel()->getCollection()->getData();
        }
        else {
            //Incremental
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
            $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
            $connection = $resource->getConnection();
            $tableName = $resource->getTableName('sales_order');
            $tableNameJobs = $resource->getTableName('filoblu_mappconnector_jobs');

            $sql = "SELECT 
                      ce.entity_id as entity_id
                    FROM
                      ".$tableName." AS ce
                    LEFT JOIN
                      ".$tableNameJobs." AS mj 
                    ON 
                      mj.entity_id = ce.entity_id
                    AND 
                      mj.entity = 'order'
                    WHERE 
                      mj.status is null";

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @return string
     */
    public function getNormalizedEntityModelPrimary() {
        return $this->_order->getEntityModelPrimary();
    }

    /**
     * Format Address
     *
     * @param $address
     * @return string
     */
    public function getFormattedAddress($address) {
        if($address) {
            $html = array();
            $street = $address->getStreet();
            foreach($street as $streetLine) {
                $html[] = $streetLine;
            }
            $html[] = $address->getPostcode();
            $html[] = $address->getCity();
            $html[] = $address->getCountry();

            $add = implode(' - ', $html);
            return rtrim($add,'- ');
        }
        else {
            return '';
        }
    }

    /**
     * Format Product Options
     *
     * @param $orderItem
     * @return mixed
     */
    public function getFormattedOptions($options) {
        $result = array();
        if (count($options)) {
            if (isset($options['options'])) {
                $result = array_merge($result, $options['options']);
            }
            if (isset($options['additional_options'])) {
                $result = array_merge($result, $options['additional_options']);
            }
            if (!empty($options['attributes_info'])) {
                $result = array_merge($options['attributes_info'], $result);
            }
        }

        $response = '';
        try {
            foreach($result as $res){
                $response = implode(', ',$res);
            }
        }
        catch(\Exception $e) {
            $this->_fileLogger->error($e->getMessage(),[],__METHOD__.':'.__LINE__);
        }        

        return $response;
    }
}